"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _htmlWebpackPlugin = _interopRequireDefault(require("html-webpack-plugin"));

var _webpack = _interopRequireDefault(require("webpack"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const PLUGIN_NAME = 'InjectInlineHtmlWebpackPlugin';

/**
 * This plugin will inject raw scripts into the head of the
 * generated HTML file from HtmlWebpackPlugin.
 *
 * ```
 * new HtmlWebpackPlugin({
 *   inlineScripts: ['my-script.js'],
 * }),
 * new InjectInlineHtmlWebpackPlugin()
 * ```
 */
class InjectInlineHtmlWebpackPlugin {
  constructor(options) {
    this.options = options ?? {};
    this.scripts = [];
  }

  apply(compiler) {
    compiler.hooks.beforeRun.tapAsync(PLUGIN_NAME, this.tapBeforeRun.bind(this));
    compiler.hooks.watchRun.tapAsync(PLUGIN_NAME, this.tapBeforeRun.bind(this));
    compiler.hooks.compilation.tap(PLUGIN_NAME, this.tapCompilation.bind(this));
  }

  getScriptKeyName(script) {
    const prefix = this.options.chunkPrefix ?? 'inject_inline';
    return `${prefix}-${this.scripts.findIndex(s => s === script)}`;
  }

  tapBeforeRun(compiler, done) {
    var _compiler$options$plu;

    // parse set of inline scripts from the html webpack configs
    const inlineScripts = (_compiler$options$plu = compiler.options.plugins) === null || _compiler$options$plu === void 0 ? void 0 : _compiler$options$plu.filter(plugin => plugin instanceof _htmlWebpackPlugin.default).map(plugin => {
      var _options;

      return ((_options = plugin.options) === null || _options === void 0 ? void 0 : _options.inlineScripts) ?? [];
    }).flat();
    this.scripts.push(...new Set(inlineScripts ?? []));

    for (const script of this.scripts) {
      new _webpack.default.EntryPlugin(process.cwd(), script, this.getScriptKeyName(script)).apply(compiler);
    }

    done();
  }

  tapCompilation(compilation) {
    const hooks = _htmlWebpackPlugin.default.getHooks(compilation);

    hooks.alterAssetTagGroups.tapAsync(PLUGIN_NAME, (data, done) => {
      var _data$plugin$options;

      const inlineScripts = [...(((_data$plugin$options = data.plugin.options) === null || _data$plugin$options === void 0 ? void 0 : _data$plugin$options.inlineScripts) ?? [])];
      this.inlineRuntime(compilation, data);
      Promise.all(inlineScripts.map(async inlineScript => {
        const assetKey = Object.keys(compilation.assets).find(key => key.startsWith(this.getScriptKeyName(inlineScript)));

        if (assetKey && compilation.assets[assetKey]) {
          const sourceCode = compilation.assets[assetKey].source(); // insert before all other scripts

          const injectOffset = (data.headTags.findIndex(tag => {
            var _tag$meta;

            return tag.tagName === 'script' && ((_tag$meta = tag.meta) === null || _tag$meta === void 0 ? void 0 : _tag$meta.plugin) !== PLUGIN_NAME;
          }) + 1 || data.headTags.length + 1) - 1;
          data.headTags.splice(injectOffset, 0, this.createInlineScriptTag(String(sourceCode)));
        }
      })).then(() => {
        done(null, data);
      });
    });
  }

  createInlineScriptTag(source) {
    return {
      tagName: 'script',
      innerHTML: `\n${source.trim()}\n`,
      voidTag: false,
      attributes: {},
      meta: {
        plugin: PLUGIN_NAME
      }
    };
  }

  inlineRuntime(compilation, data) {
    const runtimeChunkName = this.options.runtimeChunkName ?? 'runtime';
    const runtimeFilename = Object.keys(compilation.assets).find(assetName => assetName.startsWith(runtimeChunkName));
    if (!runtimeFilename) return;
    const asset = compilation.assets[runtimeFilename];
    if (!asset) return;

    const withoutRuntimeFilter = tag => {
      var _tag$attributes, _tag$attributes$src;

      return !(tag !== null && tag !== void 0 && (_tag$attributes = tag.attributes) !== null && _tag$attributes !== void 0 && (_tag$attributes$src = _tag$attributes.src) !== null && _tag$attributes$src !== void 0 && _tag$attributes$src.endsWith(runtimeFilename));
    };

    data.headTags = data.headTags.filter(withoutRuntimeFilter);
    data.bodyTags = data.bodyTags.filter(withoutRuntimeFilter);
    const runtimeTag = this.createInlineScriptTag(String(asset.source())); // insert before all other scripts

    const offset = (data.headTags.findIndex(tag => tag.tagName === 'script') + 1 || data.headTags.length + 1) - 1;
    data.headTags.splice(offset, 0, runtimeTag);
  }

}

var _default = InjectInlineHtmlWebpackPlugin;
exports.default = _default;